% TBRS model applied to Experiment 7 in Barrouillet et al., 2004

function GenB04(model)

global E   %experiment parameters
global C   %constants
global P   %model parameters
global M   %record of memory states
global O   %outcome variables for plotting

warning('off', 'MATLAB:divideByZero');  %suppresses warnings about division by zero, which occurs frequently during computation of rehearsal accuracy

rand('state',1236);  %initializes random generator
randn('state',1236); 

P.rate = 5;   %Rate R is reduced relative to default values
C.rtau = 1-exp(-P.rate*P.rehduration); %computes the threshold required to ensure that the (average) rehearsal duration = P.rehduration

% Experimental parameters
data = [2.8, 3.7, 3.6, 3.9, 4.3, 4.25, 4.5, 4.8, 5.5];  %numerals span, for decreasing CL (so increasing freetime)
bata = [4.3, 4.9, 5.3];   %baba span
E.nconds = length(data) + length(bata);
totaltime = [6, 8, 6, 10, 8, 10, 6, 8, 10, 6, 8, 10];   %first 9 = numerals, last 3 = baba
E.opnum = [12, 12, 8, 12, 8, 8, 4, 4, 4, 12, 8, 4];
nrpersec = E.opnum./totaltime;  %number retrieval per seconds
E.opduration = [ones(1,9)*0.43, ones(1,3)*0.26];  %estimate by SL from Barrouillet 07 (numerals) and from own data (baba)
E.freetime = (totaltime./E.opnum) - E.opduration; %0.0700  0.2367  0.3200  0.4033  0.5700  0.8200  1.0700  1.5700  2.0700 for numerals  
E.oprate = -log(1-C.tau)./E.opduration; %convert mean operation times into mean rates
E.enctime = 1.5;   %presentation time for an item
E.trialnum = 3;    %number of trials at each list length


fid = fopen ('TBRS.B04.out', 'w');

correct = zeros(1,E.maxstor);               %initialize correctness scoring of individual items
PC = zeros(E.maxstor, E.trialnum);          %initialize percent-correct
O.Span = zeros(E.nreplic, E.nconds);        %initialize Span
O.Serpos = zeros(E.nreplic, E.nconds, 7);   %initialize serial-position curves

for cond = 1:E.nconds
    for id = 1:E.nreplic
        Serpos = zeros(E.trialnum, E.serposval);

        %Generate vectors for position markers in context layer
        M.cue = GenContext;   

        %Generate the WM stimuli 
        M.stim = eye(C.un);

        for stor = 1:9
        for trial = 1:E.trialnum
            if E.tracing > 0, M.trace = zeros(9, 50); M.probrecall = 0; M.tpos = 1; M.time = 0;  else M.trace = []; M.time = []; end  %initialize trace

            M.w = zeros(C.cun, C.un);  %for hebbian associations to cues - reset for each trial

            %Generate list
            list = randperm(size(M.stim,1));
            list = list(1:stor);

            %Encoding and processing of distractors
            for item = 1:stor
                M.inpos = item;   %records current position in the list
                M.posfocus = item; %records on which position the focus of attention is (for rehearsal)
                GenEncoding(item, list(item), list, C.tau, P.rate, E.enctime, 1);  
                M.posfocus = 1;  %cumulative refreshing --> reset position focus to first list item
                for op = 1:E.opnum(cond)
                    GenProcess2(list, cond); 
                end
            end

            %Recall

            recall = -ones(1,9);  %0 = omission, -1 = not tested
            for probedpos = 1:stor
                M.posfocus = probedpos;
                [recall(probedpos), correct(probedpos)] = GenRecall(list, probedpos);  
            end

            %collecting data

            %collecting data
            list9 = [list, zeros(1,9-length(list))];
            PC(stor,trial) = mean(correct(1:stor));
            if stor == E.serposval, Serpos(trial, :) = correct(1:E.serposval); end

            %saving data
            fprintf (fid, '%3.0f %2.0f %1.2f %1.2f %1.0f %1.0f %1.2f   %1.2f %1.2f %1.2f %1.2f %1.2f %1.4f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f \n', ...
                id, model, E.opduration(cond), E.freetime(cond), E.opnum(cond), stor, P.cuesim, ...
                P.rate, P.ratesd, P.rehduration, P.decay, P.threshold, P.noisefactor, ...
                correct, list9, recall);

        end  %trial
        end  %stor

        %computing span

        listaccuracy = PC == 1;
        span = 0;
        for stor = 1:E.maxstor
             if ismember(1, listaccuracy(stor,:)), span = span + mean(listaccuracy(stor,:)); end
        end
        O.Span(id, cond) = span;  
        O.Serpos(id, cond, :) = mean(Serpos);

    end  %id
end %cond

SP = squeeze(mean(O.Serpos)); 
PreFigure
plot(1:7, SP(1:9,:)');
PostFigure([0,8,0,1]);
legend('12op, ft=0.07', '12op, ft=0.23', '8op, ft=0.32', '12op, ft=0.40', '8op, ft=0.57', '8op, ft=0.82', '4op, ft=1.07', '4op, ft=1.57', '4op, ft=2.07');

Meanspan = mean(O.Span);
load = E.opduration./(E.opduration + E.freetime);
num = [load(1:length(data))', nrpersec(1:length(data))', data', Meanspan(1:length(data))'];
baba = [load(length(data)+1:end)', nrpersec(length(data)+1:end)', bata', Meanspan(length(data)+1:end)'];

% plot span as a function of cognitive load 
PreFigure;
set (0, 'DefaultAxesLineStyleOrder', {':v','-v',':o','-o'}); 
plot(num(:,1), num(:, 3:4), baba(:,1), baba(:, 3:4));
legend('Data Numerals', 'Pred Numerals', 'Data Baba', 'Pred Baba');
PostFigure([0, 1, 0, 7], 'Cognitive Load', 'Span');

% plot span as a function of number of operations per second (as in Barrouillet et al., 2004) 
PreFigure;
set (0, 'DefaultAxesLineStyleOrder', {':v','-v',':o','-o'}); 
plot(num(:,2), num(:, 3:4), baba(:,2), baba(:, 3:4));
legend('Data Numerals', 'Pred Numerals', 'Data Baba', 'Pred Baba');
PostFigure([0, 2.5, 0, 7], 'Number of retrievals / Time ratio', 'Span');

fclose(fid);

end