% TBRS model applied to Brown-Peterson experiment

function GenBP(model)

global E   %experiment parameters
global C   %constants
global P   %model parameters
global M

rand('state',sum(100*clock));  %initializes random generator
randn('state',sum(100*clock)); 

% Experimental parameters
E.opduration = [0.3, 0.6];      %operation times for each processing step of the distractor task. 
E.oprate = -log(1-C.tau)./E.opduration; %convert mean operation times into mean rates
E.freetime = [0, 0.5, 1];       %free time after each distractor operation. 
E.opnum = [0, 1, 4, 8, 16];     %number of operations in the distractor period
E.enctime = 1.5;   %presentation time for an item 

if E.tracing == 0, E.setsize = 5; E.trialnum = 50; E.maxon = length(E.opnum); E.maxod = length(E.opduration); E.maxot = length(E.freetime); end  %full model
if E.tracing == 1, E.nreplic = 1; E.setsize = 5; E.trialnum = 1; E.maxon = 3; E.maxod = length(E.opduration); E.maxot = 1; end   %tracing for visualization

for i = 1:length(E.opnum)
    opnumlegtext{i} = [mat2str(E.opnum(i)), ' ops'];
end
for i = 1:length(E.opduration)
    opdurlegtext{i} = [mat2str(E.opduration(i)), ' s'];
end

fid = fopen ('TBRS.BP.out', 'w');

Serpos = zeros(E.trialnum, E.setsize); %initialize serial-position variable 
SP = zeros(E.nreplic, length(E.opnum), length(E.opduration), length(E.freetime), E.setsize);  %Serial Position summary
ACC = zeros(E.nreplic, length(E.opnum), length(E.opduration), length(E.freetime));   %Accuracy summary
    
for id = 1:E.nreplic

M.cue = GenContext; %Generate vectors for position markers in context layer  
M.stim = eye(C.un); %Generate the WM stimuli 

for on = 1:E.maxon
for od = 1:E.maxod
for ot = 1:E.maxot

for stor = E.setsize
for trial = 1:E.trialnum
if E.tracing > 0, M.trace = zeros(max(E.setsize), 50); M.tpos = 1; else M.trace = []; end  %initialize trace

M.w = zeros(C.cun, C.un);  %for hebbian associations to cues - reset for each trial

%Generate list
list = randperm(size(M.stim,1));  %shuffles the vocabulary
list = list(1:stor);

%Encoding and processing of distractors
for item = 1:stor
    M.inpos = item;
    M.posfocus = item;
    GenEncoding(item, list(item), list, C.tau, P.rate, E.enctime, 1);  
    if item == stor   %distractors only after last item
        M.posfocus = 1;  %cumulative refreshing --> reset position focus to first list item
        for op = 1:E.opnum(on)
             GenProcess(list, od, ot); 
        end
    end
end

%Recall
recall = -ones(1,9);  %0 = omission, -1 = not tested
correct = -ones(1,9); 
for probedpos = 1:stor 
    M.posfocus = probedpos;
    [recall(probedpos), correct(probedpos)] = GenRecall(list, probedpos);
end
Serpos(trial,:) = correct(1,1:E.setsize);
list9 = [list, zeros(1,9-length(list))];

%saving data
fprintf (fid, '%3.0f %2.0f %1.2f %1.2f %1.0f %1.0f %1.2f   %1.2f %1.2f %1.2f %1.2f %1.2f %1.4f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f \n', ...
    id, model, E.opduration(od), E.freetime(ot), E.opnum(on), stor, P.cuesim, ...
    P.rate, P.ratesd, P.rehduration, P.decay, P.threshold, P.noisefactor, ...
    correct, list9, recall);

end  %trial
end  %stor

SP(id, on, od, ot, :) = mean(Serpos,1);  %collecting serial position data
ACC(id, on, od, ot) = mean(SP(id,on,od,ot,:));

%plots of traces
if E.tracing == 1
    PreFigure(1, [], 2);
    plot(M.trace(:,1:M.tpos-1)');
    title(['Strength, opnum = ', mat2str(E.opnum(on)), ' opdur = ', mat2str(E.opduration(od)), ' freetime = ', mat2str(CS.freetime(ot))]);
    PostFigure;
    PreFigure(1, [], 2);
    plot(M.probrecall');
    title(['P(recall), opnum = ', mat2str(E.opnum(on)), ' opdur = ', mat2str(E.opduration(od)), ' freetime = ', mat2str(CS.freetime(ot))]);
    PostFigure;
end

end  %ot
end  %od
end  %on
end  %id

fclose(fid);

if E.plotting > 0
       
    %plot serial position functions (parameter = opnum) 
    plotnumber = 0;
    PreFigure;
    for ot = 1:length(E.freetime)
        for od = 1:length(E.opduration)
            plotnumber = plotnumber+1;
            PV = squeeze(mean(SP(:,:,od,ot,:),1));    
            x = 1:E.setsize;
            subplot(length(E.freetime),length(E.opduration),plotnumber);
            %plot(x, PV(1,:),'b-', x, PV(2,:),'r--', x, PV(3,:),'g:', x, PV(4,:), 'c-', x, PV(5,:), 'm:');
            plot(x, PV');
            if ot == 1 && od == 1, legend(opnumlegtext(1:length(E.opnum))); end  %plot legend only once
            title(['Freetime ', mat2str(E.freetime(ot)), '  Duration ', mat2str(E.opduration(od))]);
            PostFigure([0, E.setsize+1, 0, 1], 'Serial Position', 'Proportion Correct');
        end
    end
end  %if plotting

if E.plotting > 1
    %plot overall accuracy over number of operations by freetime and distractor similarity
    plotnumber = 0;
    PreFigure;
    for ot = 1:length(E.freetime)
        plotnumber = plotnumber+1;
        PV = squeeze(mean(ACC(:,:,:,ot),1));    
        x = E.opnum;
        subplot(length(E.freetime), 1, plotnumber);
        plot(x, PV);
        if ot == 1, legend(opdurlegtext(1:length(E.opduration))); end  %plot legend only once
        title(['Freetime ', mat2str(E.freetime(ot))]);
        PostFigure([0, max(E.opnum)+1, 0, 1], 'N ops', 'Proportion Correct');
    end
end

