% TBRS model applied to big Complex Span experiment

function GenCspan(model)

global E   %experiment parameters
global C   %constants
global P   %model parameters
global M   %record of memory states
global O   %outcome variables for plotting

clear PC; clear MeanPC;
warning('off', 'MATLAB:divideByZero');  %suppresses warnings about division by zero, which occurs frequently during computation of rehearsal accuracy

rand('state',1236);  %initializes random generator
randn('state',1236); 

% Experimental parameters
E.opduration = [0.3, 0.5, 0.7];      %times for doing easy/hard processing step. 
E.oprate = -log(1-C.tau)./E.opduration; %convert mean operation times into mean rates
E.freetime = [0, 0.1, 0.6, 1.2, 2];  %presentation times for operation at high/low rate. 
E.opnum = [0, 1, 4, 8];              %number of operations following each item
E.enctime = 1.5;   %presentation time for an item 

if E.tracing == 0, E.setsize = 1:E.maxstor; E.trialnum = 3; E.minon = 1; E.maxon = length(E.opnum); E.minod = 1; E.maxod = length(E.opduration); E.minot = 1; E.maxot = length(E.freetime); end  %full model
if E.tracing == 1, E.nreplic = 1; E.setsize = 7; E.trialnum = 1; E.minon = 2; E.maxon = 3; E.minod = 3; E.maxod = 3; E.minot = 1; E.maxot = 3; fidtrace = fopen('trace.dat', 'w'); end   %tracing for visualization

for i = 1:length(E.opnum)
    opnumlegtext{i} = [mat2str(E.opnum(i)), ' ops'];
end
for i = 1:length(E.freetime)
    freetimelegtext{i} = [mat2str(E.freetime(i)), ' s'];
end
for i = 1:length(E.opduration)
    opdurationlegtext{i} = [mat2str(E.opduration(i)), ' s'];
end

fid = fopen ('TBRS.Cspan.out', 'w');

correct = zeros(1,E.maxstor);               %initialize correctness scoring of individual items
PC = zeros(E.maxstor, E.trialnum);  %initialize percent-correct
Serpos = zeros(E.trialnum, E.serposval); %initialize serial-position variable 
O.SP = zeros(E.nreplic, length(E.opnum), length(E.freetime), length(E.opduration), E.serposval); 
Accuracy7 = zeros(E.nreplic, 4, 4);    %initialize accuracy of 7-item list
Span = zeros(E.nreplic, length(E.freetime)*length(E.opduration), length(E.opnum));         %initialize span by loadlevel
Span2 = zeros(E.nreplic, length(E.opnum), length(E.freetime), length(E.opduration));       %initialize span by opduration and freetime
O.Load = zeros(1, length(E.opduration)*length(E.freetime));                   %initialize cognitive load


for id = 1:E.nreplic
 
%Generate vectors for position markers in context layer
M.cue = GenContext;   
%Generate the WM stimuli 
M.stim = eye(C.un);

for on = E.minon:E.maxon
    loadlevel = 0;  % a variable to count up the load levels generated by the crossing of od and ot below
for od = E.minod:E.maxod
for ot = E.minot:E.maxot
loadlevel = loadlevel+1; 

for stor = E.setsize
for trial = 1:E.trialnum
if E.tracing > 0, M.trace = zeros(max(E.setsize), 50); M.probrecall = 0; M.tpos = 1; M.time = 0; else M.trace = []; M.time = []; end  %initialize trace

M.w = zeros(C.cun, C.un);  %for hebbian associations to cues - reset for each trial

%Generate list
list = randperm(size(M.stim,1));  %shuffles the vocabulary
list = list(1:stor);

%Encoding and processing of distractors
if ~isempty(M.trace), M = GetTrace(M, P, list, 0); end  %initial state (no time passing, thus 0)
for item = 1:stor
    M.inpos = item;   %records current position in the list
    M.posfocus = item; %records on which position the focus of attention is (for refreshing)
    GenEncoding(item, list(item), list, C.tau, P.rate, E.enctime, 1);  
    M.posfocus = 1;  %for cumulative refreshing --> reset position focus to first list item
    for op = 1:E.opnum(on)
        GenProcess(list, od, ot); 
    end
end

%Recall
recall = -ones(1,9);  %0 = omission, -1 = not tested
for probedpos = 1:stor 
    M.posfocus = probedpos;
    [recall(probedpos), correct(probedpos)] = GenRecall(list, probedpos);
end

%collecting data
list9 = [list, zeros(1,9-length(list))];
PC(stor,trial) = mean(correct(1:stor));
if stor == E.serposval, Serpos(trial, :) = correct(1:E.serposval); end

%saving data
fprintf (fid, '%3.0f %2.0f %1.2f %1.2f %1.0f %1.0f %1.2f   %1.2f %1.2f %1.2f %1.2f %1.2f %1.4f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f \n', ...
    id, model, E.opduration(od), E.freetime(ot), E.opnum(on), stor, P.cuesim, ...
    P.rate, P.ratesd, P.rehduration, P.decay, P.threshold, P.noisefactor, ...
    correct, list9, recall);

end  %trial
end  %stor

if E.tracing == 0
    %for plotting serpos, activation and accuracy at list-length 7
    O.SP(id, on, ot, od, :) = mean(Serpos);
    Accuracy7(id, loadlevel, on) = mean(PC(7,:));
    %computing span
    listaccuracy = PC == 1;
    span = 0;
    for stor = 1:E.maxstor
         if ismember(1, listaccuracy(stor,:)), span = span + mean(listaccuracy(stor,:)); end
    end
    Span(id, loadlevel, on) = span; 
    Span2(id, on, ot, od) = span; 
end

%computing cognitive load as defined by Barrouillet et al. 
O.Load(loadlevel) = E.opduration(od)/(E.opduration(od)+E.freetime(ot));
Loadcomposites(loadlevel,:) = [E.opduration(od), E.freetime(ot)];

%plots of traces of memory strength
if E.tracing == 1
    PreFigure(1, [], 2);
    plot(M.time, M.trace(:,1:M.tpos-1)');
    titletext = (['Number of operations = ', mat2str(E.opnum(on)), ' operation duration = ', mat2str(E.opduration(od)), ' free time = ', mat2str(E.freetime(ot))]);
    PostFigure([], 'Time (s)', 'Strength', titletext, [], 12);
    correct(1:E.setsize)
    for k = 1:M.tpos-1
        fprintf(fidtrace, '%1.0f %1.2f %1.2f %1.2f ', E.opnum(on), E.opduration(od), E.freetime(ot), M.time(k))
        for j = 1:size(M.trace,1) 
            fprintf(fidtrace, '%1.2f ', M.trace(j,k));
        end
        fprintf(fidtrace, '\n');
    end
end

end  %ot
end  %od
end  %on
end  %id
O.MSpan = squeeze(mean(Span, 1));   %memory span by load and on
O.MSpan2 = squeeze(mean(Span2, 1)); %memory span by on, ot, od
O.MRec = squeeze(mean(Accuracy7, 1));  %probability of correct recall (list length 7)

if E.plotting > 0 && E.tracing == 0  %for E.tracing == 1, summary plots make no sense

    compositematrix = [O.Load', Loadcomposites];
    compositematrix = sortrows(compositematrix);
    compositematrix = round(compositematrix*100)/100;  %cuts down to 2 decimals
    odtext = []; fttext = [];
    for z = 1:length(compositematrix)
        odt = '0.00'; ftt = '0.00';
        odt(1:length(mat2str(compositematrix(z,2),2))) = mat2str(compositematrix(z,2),2);
        ftt(1:length(mat2str(compositematrix(z,3),2))) = mat2str(compositematrix(z,3),2);
        odtext = [odtext; odt];
        fttext = [fttext; ftt];
    end
    
    for plotnum = 1:2
        maxvalue = 1; 
        y = [0.2, 0.4, 0.5];
        if plotnum == 1  %plot span (or activation) over load for real span, black-and-white, with lines
            if E.tracing == 2, DV = MAct; dv = 'Act'; else DV = O.MSpan; dv = 'Span'; maxvalue = 7; end  
            %for tracing = 2, no span is computed, so plot activation level at point where recall commences 
        elseif plotnum == 2  %plot percent correct over load
            DV = O.MRec; dv = 'P(recall)'; 
            y = [0.03, 0.06, 0.07];
        end
        plotmatrix = [O.Load', DV];
        plotmatrix = sortrows(plotmatrix);
        plotmatrix = aggregate(plotmatrix(:,1), plotmatrix); %averages the 3 instances of load = 1
        PreFigure
        plot(plotmatrix(:,1), plotmatrix(:,2:5));
        legend(opnumlegtext(1:length(E.opnum)));
        title([dv, ' over Load']);
        PostFigure([-0.1, 1.1, 0, maxvalue], 'Cognitive Load', dv, [], [], 12);
%         for z = 1:length(compositematrix)          %plots opduration and freetime for each load level below figure
%             text(plotmatrix(z,1), y(2)+mod(z,2)*y(3), odtext(z,:), 'FontSize', 12);
%             text(plotmatrix(z,1), y(1)+mod(z,2)*y(3), fttext(z,:), 'FontSize', 12);
%         end       
    end
            
end  %if plotting

if E.plotting > 1 && E.tracing == 0  %for E.tracing == 1, summary plots make no sense
       
    PreFigure;
    plot(E.freetime, squeeze(O.MSpan2(4,:,:)));
    legend(opdurationlegtext(1:length(E.opduration)));
    title('Span with 8 operations');
    PostFigure([0, max(E.freetime)+0.1, 0, 8], 'Free Time', 'Span', [], [], 12);
    
    PreFigure;
    plot(squeeze(mean(mean(mean(O.SP,3),4),1))');  %averaging over dimensions 1, 3, 4 --> leaving dimension 2 = opnum
    legend(opnumlegtext(1:length(E.opnum)));
    title(['Serial Position Curve & OpNum']);
    PostFigure([0, 8, 0, 1], 'Serial Position', 'P(recall)', [], [], 12);
    
    PreFigure;
    plot(squeeze(mean(mean(mean(O.SP(:,2:length(E.opnum),:,:,:),2),4),1))');  %leaving dimension 3 = freetime
    legend(freetimelegtext(1:length(E.freetime)));
    title(['Serial Position Curve & Free Time']);
    PostFigure([0, 8, 0, 1], 'Serial Position', 'P(recall)', [], [], 12);    
    
    PreFigure;
    plot(squeeze(mean(mean(mean(O.SP(:,2:length(E.opnum),:,:,:),2),3),1))');  %leaving dimension 4 = opduration
    legend(opdurationlegtext(1:length(E.opduration)));
    title(['Serial Position Curve & Operation Duration']);
    PostFigure([0, 8, 0, 1], 'Serial Position', 'P(recall)', [], [], 12);      
    
end

fclose(fid);

end