% TBRS model applied to Barrouillet et al 2009 Child Development E2

function GenCspanKids(model)

global E   %experiment parameters
global C   %constants
global P   %model parameters
global M   %record of memory states

nage = 2;  %number of age groups modelled
attcapture = 1.0; %proportion of digit reading time during which attention is captured
devmodel = 7;   %Developmental model tested --> row of parameter values
                %Model 7 is the model published in PB&R

%parameter values for young kids, old kids, and adults (in that order from left to right; different rows = different developmental models
Rate = [3, 5,   6;       %devmodel 1: age differences in rate only
        3, 4.5, 6;       %devmodel 2: age differences in rate and strength of encoding (tau)
        6, 6,   6;       %devmodel 3: age differences in decay
        6, 6,   6;       %devmodel 4: age differences in noise
        3, 4,   6;       %devmodel 5: age differences in rate, tau, and decay (as per B09, p. 488)
        2.5, 5, 6;       %devmodel 6: age differences in rate and tau
        6, 6,   6;       %devmodel 7: age differences in decay and tau
        6, 6,   6;       %devmodel 8: age differences in ratesd only
        3, 5,   6;       %devmodel 9: age differences in rate and decay
        6, 6,   6;       %devmodel 10: age differences in refreshing probability
        5.5, 5.5,  5.5];      %devmodel 11: age differences in decay (and ?), with ratesd = 2.5 (or larger)
    
Decay = [0.5, 0.5, 0.5;
         0.5, 0.5, 0.5;
         0.5, 0.4, 0.5;
         0.5, 0.5, 0.5;
         0.5, 0.4, 0.4;
         0.4, 0.4, 0.4;
         0.5, 0.4, 0.4;
         0.5, 0.5, 0.5;
         0.5, 0.4, 0.5;
         0.5, 0.5, 0.5;
         0.65, 0.4, 0.5];

Ratesd = [1.0, 1.0, 1.0;
          1.0, 1.0, 1.0;
          1.0, 1.0, 1.0;
          1.0, 1.0, 1.0;
          1.0, 1.0, 1.0;
          1.0, 1.0, 1.0;
          1.0, 1.0, 1.0;          
          3.5, 2.5, 1.0;
          1.0, 1.0, 1.0;
          1.0, 1.0, 1.0;
          2.5, 2.5, 1.0];

Rprob = [1.0, 1.0, 1.0;
         1.0, 1.0, 1.0;
         1.0, 1.0, 1.0;
         1.0, 1.0, 1.0;
         1.0, 1.0, 1.0;
         1.0, 1.0, 1.0;
         1.0, 1.0, 1.0;
         1.0, 1.0, 1.0;
         1.0, 1.0, 1.0;
         0.5, 0.8, 1.0;
         1.0, 1.0, 1.0];

Tau = [0.95, 0.95, 0.95;
       0.50, 0.75, 0.95;
       0.95, 0.95, 0.95;
       0.95, 0.95, 0.95;
       0.50, 0.55, 0.95;
       0.50, 0.55, 0.95;
       0.50, 0.55, 0.95;
       0.95, 0.95, 0.95;
       0.95, 0.95, 0.95;
       0.95, 0.95, 0.95;
       0.95, 0.95, 0.95];  

Noise = [1, 1, 1;   %expressed as factor multiplied with standard noise parameter
         1, 1, 1;
         1, 1, 1;
         3, 2, 1;
         1, 1, 1;
         1, 1, 1;
         1, 1, 1;
         1, 1, 1;
         1, 1, 1;
         1, 1, 1;
         1, 1, 1];
   
warning('off', 'MATLAB:divideByZero');  %suppresses warnings about division by zero, which occurs frequently during computation of rehearsal accuracy
rand('state',1234);  %initializes random generator
randn('state',1234);

% Experimental parameters
Opduration = [0.622 0.622 0.622;
              0.489 0.489 0.489;
              0.430 0.430 0.430];   %times for doing processing steps (first 3 for 8y old, next 3 for 14y old, last 3 for adults from B04)
Opduration = Opduration*attcapture;
Totaltime = [0.622, 1.244, 2.488;
             0.489, 0.978, 1.956;
             0.500, 0.750, 1.500];
Freetime = Totaltime - Opduration;  %free times after each operation
Opnum = [12, 6, 3;
         12, 6, 3;
         12, 8, 4];                 %number of operations following each item
E.enctime = 1;                      %presentation time for an item
E.serposval = 5;  %list length for SPCs

if E.tracing == 0, E.setsize = 1:E.maxstor; E.trialnum = 3; E.minon = 1; E.maxon = length(Opnum); E.maxod = 6; E.minot = 1; E.maxot = 6; end  %full model
if E.tracing == 1, E.nreplic = 1; E.setsize = 4; E.trialnum = 1; E.minon = 3; E.maxon = 3; E.maxod = 1; E.minot = 1; E.maxot = 1; end   %tracing for visualization

B09Spandata = [2.35, 1.70, 1.39;   %data from Barrouillet et al. 09, p. 483
               3.71, 2.75, 2.17];
B04Spandata = [4.5, 3.6, 2.8];     %data points 9, 7, and 3 (starting from lowest to highest load) from Exp. 7 of Barrouillet et al., 2004, as adult comparison data
           
fid = fopen ('TBRS.B09.out', 'w');

correct = zeros(1,E.maxstor);               %initialize correctness scoring of individual items
PC = zeros(E.maxstor, E.trialnum);  %initialize percent-correct
Serpos = zeros(E.trialnum, E.serposval); %initialize serial-position variable
SP = zeros(E.nreplic, nage, length(Opnum), E.serposval);
Accuracy = zeros(E.nreplic, nage, length(Opnum));    %initialize accuracy of list at a fixed length
Span = zeros(E.nreplic, nage, length(Opnum));         %initialize span
Load = zeros(nage, length(Opnum));                   %initialize cognitive load

for age = 1:nage  %1 = younger kids, 2 = older kids, 3 = adults from B04
if age == 3, E.enctime = 1.5; end  %encoding time was longer in B04  

P.rate = Rate(devmodel, age);       
P.decay = Decay(devmodel, age);
P.ratesd = Ratesd(devmodel, age);
P.rprob = Rprob(devmodel, age);
P.noisefactor = P.noisefactor*Noise(devmodel, age);
C.tau = Tau(devmodel, age);
E.opduration = Opduration(age,:);
E.freetime = Freetime(age,:);
E.opnum = Opnum(age,:);
C.rtau = 1-exp(-P.rate*P.rehduration);      %computes the threshold required to ensure that the (average) refreshing duration = P.rehduration
E.oprate = -log(1-C.tau)./E.opduration;     %convert mean operation times into mean rates
for i = 1:length(E.opnum)
    opnumlegtext{i} = [mat2str(E.opnum(i)), ' ops'];
end


for id = 1:E.nreplic

    %Generate vectors for position markers in context layer
    M.cue = GenContext;

    %Generate the WM stimuli
    M.stim = eye(C.un);

    for cond = 1:length(E.opduration)

        for stor = E.setsize
            for trial = 1:E.trialnum
                if E.tracing > 0, M.trace = zeros(max(E.setsize), 50); M.probrecall = 0; M.tpos = 1; M.time = 0; else M.trace = []; M.time = []; end  %initialize trace

                M.w = zeros(C.cun, C.un);  %for hebbian associations to cues - reset for each trial

                %Generate list
                list = randperm(size(M.stim,1));
                list = list(1:stor);

                %Encoding and processing of distractors
                if ~isempty(M.trace), M = GetTrace(M, P, list, 0); end  %initial state (no time passing, thus 0)
                for item = 1:stor
                    M.inpos = item;   %records current position in the list
                    M.posfocus = item; %records on which position the focus of attention is (for rehearsal)
                    GenEncoding(item, list(item), list, C.tau, P.rate, E.enctime, 1);
                    M.posfocus = 1;  %cumulative refreshing --> reset position focus to first list item
                    for op = 1:E.opnum(cond)
                        opcorrect(item, op) = GenProcess2(list, cond);
                    end
                end

                %Recall
                recall = -ones(1,9);
                for probedpos = 1:stor 
                    M.posfocus = probedpos;
                    [recall(probedpos), correct(probedpos)] = GenRecall(list, probedpos);
                end

                %collecting data
                list9 = [list, zeros(1,9-length(list))];
                PC(stor,trial) = mean(correct(1:stor));
                if stor == E.serposval, Serpos(trial, :) = correct(1:E.serposval); end
                
                %saving data
                fprintf (fid, '%3.0f %2.0f %1.2f %1.2f %1.0f %1.0f %1.2f   %1.2f %1.2f %1.2f %1.2f %1.2f %1.4f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f \n', ...
                    age, model, E.opduration(cond), E.freetime(cond), E.opnum(cond), stor, P.cuesim, ...
                    P.rate, P.ratesd, P.rehduration, P.decay, P.threshold, P.noisefactor, ...
                    correct, list9, recall);

            end  %trial
        end  %stor

        if E.tracing ~= 1 
            %for plotting serpos, activation and accuracy at list-length = E.serposval
            SP(id, age, cond, :) = mean(Serpos);
            Accuracy(id, age, cond) = mean(PC(E.serposval,:));
        end

        %computing span
        if E.tracing < 2
            listaccuracy = PC == 1;
            span = 0;
            for stor = 1:E.maxstor
                if ismember(1, listaccuracy(stor,:)), span = span + mean(listaccuracy(stor,:)); end
            end
            Span(id, age, cond) = span;
        end

        %computing cognitive load as defined by Barrouillet et al.
        Load(age, cond) = E.opduration(cond)/(E.opduration(cond)+E.freetime(cond));
        
        %plots of traces of memory strength
        if E.tracing == 1
            PreFigure(1, [], 2);
            plot(M.time, M.trace(:,1:M.tpos-1)');
            titletext = (['Number of operations = ', mat2str(E.opnum(cond)), ' operation duration = ', mat2str(E.opduration(cond)), ' free time = ', mat2str(E.freetime(cond))]);
            PostFigure([], 'Time (s)', 'Strength', titletext, [], 12);
            correct(1:E.setsize)
        end

    end  %cond
end  %id
end %age
MSpan = squeeze(mean(Span, 1));   %memory span

fclose(fid);


if E.plotting > 0 && E.tracing == 0  %for E.tracing == 1, summary plots make no sense

    if nage == 2
        PreFigure;
        set (0, 'DefaultAxesLineStyleOrder', {'-o','-v',':o',':v'}); 
        plotmatrix = [Load', MSpan'];  
        plotmatrix = sortrows(plotmatrix);  %this works only if the cognitive load levels are sorted in the same way for all age groups!
        plot(plotmatrix(:,1), plotmatrix(:,3), plotmatrix(:,2), plotmatrix(:,4), ...
            plotmatrix(:,1), B09Spandata(1,:)', plotmatrix(:,2), B09Spandata(2,:)');
        legend('pred: 8y', 'pred: 14y', 'data: 8y', 'data: 14y');
        title('Span over Load');
        PostFigure([-0.1, 1.1, 0, 7], 'Load', 'Span');
    end
    if nage == 3
        PreFigure;
        set (0, 'DefaultAxesLineStyleOrder', {'-o','-v','-s',':o',':v',':s'}); 
        plotmatrix = [Load', MSpan'];  
        plotmatrix = sortrows(plotmatrix);  %this works only if the cognitive load levels are sorted in the same way for all age groups!
        plot(plotmatrix(:,1), plotmatrix(:,4), plotmatrix(:,2), plotmatrix(:,5), plotmatrix(:,3), plotmatrix(:,6), ...
            plotmatrix(:,1), B09Spandata(1,:)', plotmatrix(:,2), B09Spandata(2,:)', plotmatrix(:,3), B04Spandata');
        legend('pred: young', 'pred: older', 'pred: adults', 'data: young', 'data: older', 'data: adults');
        title('Span over Load');
        PostFigure([-0.1, 1.1, 0, 7], 'Load', 'Span');
    end

end  %if plotting

if E.plotting > 1 && E.tracing == 0  %for E.tracing == 1, summary plots make no sense

    PreFigure;
    plot(squeeze(mean(SP(:,1,:,:)))');  
    legend(opnumlegtext(1:length(E.opnum)));
    title('Serial Position Curve & OpNum/Load, 8 year olds');
    PostFigure([0, E.serposval+1, 0, 1], 'Serial Position', 'P(recall)');
    
    PreFigure;
    plot(squeeze(mean(SP(:,2,:,:)))');  
    legend(opnumlegtext(1:length(E.opnum)));
    title('Serial Position Curve & OpNum/Load, 14 year olds');
    PostFigure([0, E.serposval+1, 0, 1], 'Serial Position', 'P(recall)');
    
    if nage == 3
        PreFigure;
        plot(squeeze(mean(SP(:,3,:,:)))');  
        legend(opnumlegtext(1:length(E.opnum)));
        title('Serial Position Curve & OpNum/Load, adults');
        PostFigure([0, E.serposval+1, 0, 1], 'Serial Position', 'P(recall)');
    end

end

