% TBRS model applied to Complex Span experiment with 1 distractor, 4
% identical distractors, 3 different distractors (Experiment 3 of
% Lewandowsky, Geiger, et al., 2010)

function GenCspanTiming(model)

global E   %experiment parameters
global C   %constants
global P   %model parameters
global M   %record of memory states
global O   %outcome variables for plotting

clear PC; clear MeanPC;
warning('off', 'MATLAB:divideByZero');  %suppresses warnings about division by zero, which occurs frequently during computation of rehearsal accuracy

rand('state',1236);  %initializes random generator
randn('state',1236); 

Time = [repmat([1.15, 0.43, 0.43, 0.43, 0.43, 0.43, 0.43, 0.43],3,1);
               [1.15, 0.64, 0.64, 0.64, 0.64, 0.64, 0.64, 0.64]];      %measured speaking durations (plus inter-speech-pauses) for E3 in Lsky et al (2010)
Attcapture = 3;    %assumption about attentional capture: 1 = all words equal; 2 = first word more, later identical words a little; 3 = first words only, later identical words none at all
if Attcapture == 1
    Opduration = Time - 0.2; 
end
if Attcapture == 2
    P.noisefactor = 0.05;                                    %increase noise to get accuracy closer to data
    Opduration = [repmat([0.7, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1],3,1);    %times for consecutive operations in a simple burst
                         [0.7, 0.3, 0.3, 0.3, 0.3, 0.3, 0.3, 0.3]];        %times for consecutive operations in a complex burst 
end
if Attcapture == 3
    P.noisefactor = 0.05;                                    %increase noise to get accuracy closer to data
    Opduration = [repmat([0.7, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],3,1);    %times for consecutive operations in a simple burst
                         [0.7, 0.3, 0.3, 0.3, 0.3, 0.3, 0.3, 0.3]];        %times for consecutive operations in a complex burst 
end
Freetime = Time-Opduration;   
E.opnum = [0, 1, 4, 3]; 
nconds = 4;


% Other experimental parameters
E.setsize = 5; stor = E.setsize;
E.enctime = 1.0;   %presentation time for an item 

if E.tracing == 0, E.trialnum = 20; end  %full model
if E.tracing == 1, E.trialnum = 1; end   %tracing for visualization

fid = fopen ('TBRS.CspanTiming.out', 'w');

correct = zeros(1,E.maxstor);           %initialize correctness scoring of individual items
Serpos = zeros(E.trialnum, E.setsize); %initialize serial-position variable 
O.SP = zeros(E.nreplic, nconds, E.setsize); 

for id = 1:E.nreplic
 
%Generate vectors for position markers in context layer
M.cue = GenContext;   
%Generate the WM stimuli 
M.stim = eye(C.un);

for cond = 1:nconds
    E.opduration = Opduration(cond,:);
    E.oprate = -log(1-C.tau)./E.opduration; %convert mean operation times into mean rates
    E.freetime = Freetime(cond,:);
for on = 1:length(E.opnum)
for trial = 1:E.trialnum
if E.tracing > 0, M.trace = zeros(max(E.setsize), 50); M.probrecall = 0; M.tpos = 1; M.time = 0; else M.trace = []; M.time = []; end  %initialize trace

M.w = zeros(C.cun, C.un);  %for hebbian associations to cues - reset for each trial

%Generate list
list = randperm(size(M.stim,1));  %shuffles the vocabulary
list = list(1:stor);

%Encoding and processing of distractors
if ~isempty(M.trace), M = GetTrace(M, P, list, 0); end  %initial state (no time passing, thus 0)
for item = 1:stor
    M.inpos = item;   %records current position in the list
    M.posfocus = item; %records on which position the focus of attention is (for refreshing)
    GenEncoding(item, list(item), list, C.tau, P.rate, E.enctime, 1);  
    M.posfocus = 1;  %for cumulative refreshing --> reset position focus to first list item
    for op = 1:E.opnum(cond)
        GenProcess(list, op, op);  %uses op as index for E.opduration and E.freetime within the function 
    end
end

%Recall

recall = -ones(1,9);  %0 = omission, -1 = not tested
correct = -ones(1,9);
for probedpos = 1:stor 
    M.posfocus = probedpos;
    [recall(probedpos), correct(probedpos)] = GenRecall(list, probedpos);
end

%collecting data
list9 = [list, zeros(1,9-length(list))];
Serpos(trial, :) = correct(1:E.setsize); 

%saving data
fprintf (fid, '%3.0f %2.0f %1.2f %1.2f %1.0f %1.0f %1.2f   %1.2f %1.2f %1.2f %1.2f %1.2f %1.4f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f \n', ...
    id, model, E.opduration(cond), E.freetime(cond), E.opnum(cond), cond, P.cuesim, ...
    P.rate, P.ratesd, P.rehduration, P.decay, P.threshold, P.noisefactor, ...
    correct, list9, recall);

end  %trial

O.SP(id, cond, :) = mean(Serpos);

%plots of traces of memory strength
if E.tracing == 1
    PreFigure(1, [], 2);
    plot(M.time, M.trace(:,1:M.tpos-1)');
    titletext = (['Number of operations = ', mat2str(E.opnum(on))]);
    PostFigure([], 'Time (s)', 'Strength', titletext, [], 12);
    correct(1:E.setsize)
end

end  %on
end  %cond
end  %id

if E.plotting > 0 && E.tracing == 0  %for E.tracing == 1, summary plots make no sense

    PreFigure
    plotvector = squeeze(mean(O.SP,1))';  %averaging over id
    plot(plotvector);  
    title('Serial Position Curves');
    legend('0 ops', '1 op', '4 ops simple', '3 ops complex');
    PostFigure([0, E.setsize+1, 0, 1], 'Serial Position', 'P(recall)');
   
end

fclose(fid);

end