% TBRS model applied to Brown-Peterson experiment (Liefhooge et al., 2008,
% Expt 4)

function GenLief(model)

global E   %experiment parameters
global C   %constants
global P   %model parameters
global M

rand('state',sum(100*clock));  %initializes random generator
randn('state',sum(100*clock));
LL= [0 0 0 1 0 2 0 3];  %pointers from list lengths to results array

% Experimental parameters
E.opduration = [0.73, 0.77 .78];      %times for doing each processing step of the distractor task.
E.oprate = -log(1-C.tau)./E.opduration; %convert mean operation times into mean rates
E.freetime = [.47, .43, .42];       %free time after each distractor operation (adds to 1.2 sec with opdur).
E.opnum = 8;     %number of operations in the distractor period
E.enctime = 1.8;   %presentation time for an item (1.5s plus .3 ISI)

if E.tracing == 0, E.setsize = [4 6 8]; E.trialnum = 4; E.maxon = length(E.opnum); E.maxod = length(E.opduration); E.maxot = length(E.freetime); end  %full model
if E.tracing == 1, E.nreplic = 1; E.setsize = 5; E.trialnum = 1; E.maxon = 3; E.maxod = length(E.opduration); E.maxot = 1; end   %tracing for visualization

for i = 1:length(E.opnum)
    opnumlegtext{i} = [mat2str(E.opnum(i)), ' ops'];
end
for i = 1:length(E.opduration)
    opdurlegtext{i} = [mat2str(E.opduration(i)), ' s'];
end

fid = fopen ('TBRS.Lief.out', 'w');

ACC = zeros(E.nreplic, length(E.setsize), length(E.opduration));  %initializes variable for accuracies

for id = 1:E.nreplic

    M.cue = GenContext; %Generate vectors for position markers in context layer
    M.stim = eye(C.un); %Generate the WM stimuli

    for on = 1:E.maxon
        for od = 1:E.maxod
            ot = od;   %opduration and free time not fully crossed

            for stor = E.setsize
                Serpos = zeros(E.trialnum, stor); %initialize serial-position variable
                for trial = 1:E.trialnum
                    if E.tracing > 0, M.trace = zeros(max(E.setsize), 50); M.tpos = 1; else M.trace = []; end  %initialize trace

                    M.w = zeros(C.cun, C.un);  %for hebbian associations to cues - reset for each stimulus set

                    %Generate list
                    list = randperm(size(M.stim,1));  %shuffles the vocabulary
                    list = list(1:stor);

                    %Encoding and processing of distractors
                    for item = 1:stor
                        M.inpos = item;
                        M.posfocus = item;
                        GenEncoding(item, list(item), list, C.tau, P.rate, E.enctime, 1);
                        if item == stor   %distractors only after last item
                            M.posfocus = 1;  %cumulative refreshing --> reset position focus to first list item
                            for op = 1:E.opnum(on)
                                GenProcess(list, od, ot);
                            end
                        end
                    end
                    if ~isempty(M.trace), Act(stor,trial) = mean(M.trace(1:stor,M.tpos-1)); end

                    %Recall
                    recall = -ones(1,9);  %0 = omission, -1 = not tested
                    correct = -ones(1,9);
                    for probedpos = 1:stor
                        M.posfocus = probedpos;
                        [recall(probedpos), correct(probedpos)] = GenRecall(list, probedpos);
                    end
                    Serpos(trial,:) = correct(1,1:stor);
                    list9 = [list, zeros(1,9-length(list))];

                    %saving data
                    fprintf (fid, '%3.0f %2.0f %1.2f %1.2f %1.0f %1.0f %1.2f   %1.2f %1.2f %1.2f %1.2f %1.2f %1.4f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f   %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f %1.0f \n', ...
                        id, model, E.opduration(od), E.freetime(ot), E.opnum(on), stor, P.cuesim, ...
                        P.rate, P.ratesd, P.rehduration, P.decay, P.threshold, P.noisefactor, ...
                        correct, list9, recall);

                end  %trial

                ACC(id, LL(stor), od) = mean(mean(Serpos,1));
            end  %stor

            %plots of traces
            if E.tracing == 1
                PreFigure(1, [], 2);
                plot(M.trace(:,1:M.tpos-1)');
                title(['Strength, opnum = ', mat2str(E.opnum(on)), ' opdur = ', mat2str(E.opduration(od)), ' freetime = ', mat2str(E.freetime(ot))]);
                PostFigure;
                PreFigure(1, [], 2);
                plot(M.probrecall');
                title(['P(recall), opnum = ', mat2str(E.opnum(on)), ' opdur = ', mat2str(E.opduration(od)), ' freetime = ', mat2str(E.freetime(ot))]);
                PostFigure;
            end
        end  %od
    end  %on
end  %id

fclose(fid);

%print mean accuracies as in Table 10 in Liefhooge et al.
acm=squeeze(mean(ACC,1))'



