% Wrapper for TBRS model - version for publication
 clear all;    

global E   %experiment parameters
global C   %constants
global P   %model parameters
global O   %outcome variables

experiment = 1;    %1 = Big Complex Span (Simulations 1 and 3 in O&L, PB&R); %2 = Barrouillet et al. 2004, Exp. 7; 
                   %3: Barrouillet et al. 2009, Exp. 2; 4 = Lewandowsky, Geiger, et al. (2010), Exp. 3;
                   %5 = Liefooghe et al., 2008, Exp. 4; 6: Generic Brown-Peterson 

% General experimental parameters (common to all simulations) and parameters for controlling simulation
E.maxstor = 9;       %maximum list length (for span procedure) 
E.plotting = 2;      %0 = none, 1 = minimalist, 2 = more (depending on experiment)
E.tracing = 0;       %0 = off, 1 = tracing for visualization; 
E.serposval = 7;     %list length for serial position curve plots
E.nreplic = 200;     %default, can be overwritten in later functions (depending on tracing)

%Constants
C.un = 81;         %number of units for stimuli (total number in the stimulus layer)
C.cn = 6;          %number of units for each cue (= position marker)
C.tau = 0.95;      %threshold of learning strength at which encoding / processing finishes
C.rectime = 0.5;   %time for recalling an item
C.rstrategy = 1;   %0 = no refreshing during presentation time of items; 1 = use remainder of presentation time of items for refreshing
C.rschedule = 1;   %1 = start over at beginning of list after each operation; 2 = continue after each operation, start over at beginning after each new item

%    cuesim rate ratesd rduration decay threshold noise 
V = [ 0.3   6    1      0.08      0.5    0        0;    % 1: TBRS-zero: no threshold, no noise
      0.3   6    1      0.08      0.5    0.1      0;    % 2: model with threshold
      0.3   6    1      0.08      0.5    0        0.02; % 3: model with  noise
      0.3   6    1      0.08      0.5    0.05     0.02];% 4: TBRS*: model with threshold and noise
       
msel =  [4];     %specify one or more models by selecting a row of V parameters

for m = 1:length(msel)
    
    model = msel(m);
    P.cuesim = V(model, 1);
    P.rate = V(model, 2);
    P.ratesd = V(model, 3);
    P.rehduration = V(model, 4);
    P.decay = V(model, 5);
    P.threshold = V(model, 6);
    P.noisefactor = V(model, 7);
    P.rprob = 1;  %default value for refreshing probability
    P.recallrate = -log(1-C.tau)/C.rectime; %computes the recall rate needed to achieve average time of C.rectime
    C.rtau = 1-exp(-P.rate*P.rehduration); %computes the threshold required to ensure that the (average) refreshing duration = P.rehduration

    if experiment == 1, GenCspan(model); end
    if experiment == 2, GenB04(model); end
    if experiment == 3, GenCspanKids(model); end
    if experiment == 4, GenCspanTiming(model); end
    if experiment == 5, GenLief(model); end
    if experiment == 6, GenBP(model); end
    
end %for model
